README_CONTENTS = """usrsvc
======

A user service manager. Intended for managing services/daemons running as service account users.



Design
------

usrsvc comes in two parts, *usrsvc* (tool to start/stop/restart/get status of services) and *usrsvcd* (daemon which keeps services running, monitors running services, etc). 

You may use *usrsvc* standalone, or in conjunction with *usrsvcd*.


**No Parent/Child**

Contrary to other service managers, such as supervisord, usrsvcd does NOT require itself be the parent of the processes it manages. This allows usrsvcd to be restarted, updated, or not run at all, without affecting the services it manages.

In the supervisord world, if you need to update the running supervisord, you must also restart all processes that it is managing which causes an unnecessary outage.

All services started by *usrsvc* or *usrsvcd* have "init" as the parent (or systemd under some system configurations).


**Capturing/Managing existing processes**

Because no parent-child relationship is required or even used, usrsvc is able to manage already-running processes. There are several configurable means for usrsvc to identify and match the services you define to running processes.


**Modular / Resilient**


Instead of one monolithic daemon, usrsvc is broken into *usrsvc* (tool to start/stop/restart/status), daemon *usrsvcd* (tool to manage autostart, autorestart, monitoring), and in the future an XLMRPC daemon for a web interface.

Each tool does exactly just what it needs to do, and is designed to recover from any failure scenario and keep going. 

You can, for example, run "usrsvcd reread" (or send USR1 signal to usrsvcd process) to have it attempt to reread the configuration. If there is an error with the new configuration, it will be logged, and the old configuration retained. 
If the new configuration passes validation, the changes will be applied to the next set of operations performed by usrsvcd. Unlike other service managers, *usrsvcd* will not crash with a configuration error.


Several commands / log entries give a JSON output so they can be parsed and used by an external application.


**Process Monitoring**


*usrsvcd* provides the ability to monitor the processes it manages beyond just whether it is stopped/started. These are configurable, optional, and defined per-service (see "Monitoring" section below.)


**No root / Isolation**

usrsvc is designed to run at the service account level, per best-practices of security isolation. It requires no root configuration or service to be running; everything is isolated to the user account running the service (like "web" or "django")

Each account has its own independent configurations, and runs its own instance of *usrsvcd*, providing further isolation between application groups.


**Simple Configuration**

All of the configuration is through simple ini-style config files, and supports defining and inheriting default settings to prevent duplication across programs.

**Verbose**

Usrsvc and usrsvcd are very verbose with logging, and try to be as specific as possible. All logs contain timestamps and meaningful error codes/descriptions, to simplify and even make possible evaluation of issues with your services.

usrsvc (tool)
-------------


The "usrsvc" tool handles the basic operations of starting/stopping/restarting/status of a service. You can use this with or without *usrsvcd* running to manage services.

	Usage: usrsvc (Options) [start/stop/restart/status] [program name]

		Performs the requested action on the given program name.

		"all" can be used for start/stop/restart in place of "program name"

	 
	usrsvc is tool for performing specific actions on services, usrsvcd is the related daemon for autorestart/monitoring, etc.


	Options:

	\-\-\-\-\-\-\-\-


		Parallel:

			When doing start/stop/restart all, you may add "\-\-parallel" to perform 

			the action on all items in parallel.

			  

	Config:

	\-\-\-\-\-\-\-


		Usrsvc uses the config file found at $HOME/usrsvc.cfg (/home/media/usrsvc.cfg).


	Documentation

	\-\-\-\-\-\-\-\-\-\-\-\-\-


		Run "usrsvc \-\-readme" or see https://github.com/kata198/usrsvc/blob/master/README.md 

		  for more documentation.


The tool will output some basic information about what happened, and give a meaningful return code (0 = success, otherwise see https://raw.githubusercontent.com/kata198/usrsvc/master/usrsvcmod/constants.py "ReturnCodes" object for the list of return codes used and descriptions).

Usrsvc will be as verbose as possible in identifying why a program failed to start and stay running, to ease debugging.

**Example Usage**

start:

	[myuser]$ usrsvc start MagicLooper

	[Tue Mar  8 22:14:34 2016] - Started MagicLooper:


	{'args': ['/home/svcact/bin/MagicLooper.py'], 'cmdline': '/usr/bin/python /home/svcact/bin/MagicLooper.py', 'pid': 12467, 'executable': '/usr/bin/python', 'running': True}



status:

	[myuser]$ usrsvc status MagicLooper

	[Tue Mar  8 22:14:55 2016] - MagicLooper is running:


	{'args': ['/home/svcact/bin/MagicLooper.py'], 'cmdline': '/usr/bin/python /home/svcact/bin/MagicLooper.py', 'pid': 12467, 'executable': '/usr/bin/python', 'running': True}


stop:

	[myuser]$ usrsvc stop MagicLooper

	[Tue Mar  8 22:15:37 2016] - Stopping MagicLooper [12467]

	[Tue Mar  8 22:15:37 2016] - MagicLooper terminated


usrsvcd (daemon)
----------------

The *usrsvcd* daemon handles the autostart, autorestart, and monitoring of the configured services. It is optional, and not required to use usrsvc, but required for advanced features.

	Usage: usrsvcd (Optional: [action])

	Optional daemon portion of usrsvc which actively monitors processes and provides the autostart/autorestart, and other optional features.


		Actions:

			Running with no arguments starts the usrsvcd daemon. You can also provide one of several "action" arguments which affect the running instance of usrsvcd.


			checkconfig            -   Try to parse config files and validate correctness, without affecting the running usrsvcd instance. Returns non-zero on failure.

			reread                 -   Sends SIGUSR1 to the running usrsvcd process, which will cause it to reread configs and immediately apply the changes to the running instance.

										If there are errors in the configs, a message will be logged by the usrsvcd process and it will retain its current configuration state.

			restart                -   Restarts the usrsvcd daemon cleanly

			status                 -   Checks if usrsvcd is running. Returns non-zero on failure

			stop                   -   Stops running instance of usrsvcd


	Uses main config file in $HOME/usrsvc.cfg



The *usrsvcd* process when started will pick up the state of any configured services (whether they are running, what their pid is, etc), it does not need to start/restart the processes to manage them.

**Reread config live**

If you want to add/remove/change a service or a service's properties, you can do so by updating the configuration files, and then optionally running "usrsvcd checkconfig" (to validate config and give you errors on stderr), followed by "usrsvcd reread".

If there are errors in the configs, they will be logged and the previous configuration will be kept. Otherwise, after the current operation set is completed, *usrsvcd* will update all its internal references to the new config, and continue with them.

There is no need to restart usrsvcd or any of its services to apply a config change in this regard.


Process Identification
----------------------

Processes are managed through pidfiles. Every program has a required pidfile field in its configuration.

By default (autopid=True), a pidfile is generated by usrsvc after starting your service. If you set autopid to False, you will need to generate the pidfile.

The process id found within the pidfile has its proctitle validated, to ensure it is the expected application.

By default, a regular expression is generated from the command given to match the proctitle. If your application changes its proctitle, you will have to defined *proctitle_re* with a matching expression.


If a pidfile is not found, or does not match, by default (when scan_for_process is True), all running processes by the same user will be scanned for one with a proctitle match. If a match is found, the process has been identified and a pidfile will be generated.


Configuration
=============


Configuration starts with the "main" config at $HOME/usrsvc.cfg . This file defines some basic info, or can contain your full configuration if you want. The recommended usage is to provide the "config_dir" property therein, which specifies a directory. In that directory, all files ending in ".cfg" will be processed, allowing you to have each Program defined in its own config, default settings in another config, etc. This makes it simpler to manage and add/remove services.


Configuration is "configobj" style, which closely mimics ini-style but supports subsections.

The following are the sections and their meanings. [Main] must be defined in $HOME/usrsvc.cfg, but otherwise any of the sections can appear in any config file.


**[Main]**

The [Main] section must be found in $HOME/usrsvc.cfg, and can contain any of the following properties:


* config_dir - This defines a directory which will be searched for additional configuration. Anything with a ".cfg" suffix will be processed as a config.

* pidfile - REQUIRED - This defines the location where *usrsvcd* will store its own pid.

* usrsvcd_stdout - If defined, usrsvcd will log stdout to this file instead of the default stdout (likely a terminal). Must be an absolute path.

* usrsvcd_stderr - If defined, usrsvcd will log stderr to this file instead of the default stderr (likely a terminal). Use the value "stdout" to log stderr to the same location as stdout, otherwise must be an absolute path.


**[Program:myprogram]**

Each "Program" section can be in any config file, and defines a Program that will be managed by usrsvc. Following the colon is the program name (in this case, "myprogram") and must be unique. This will assign the name that will be used to identify the program (e.x. "usrsvc start myprogram")


The "Program" section has the following properties:


* command - REQUIRED - Full command and arguments to execute. If #useshell# is True, this can contain shell-isms

* useshell - If True, will invoke your application through a shell. You can use shell expressions in this mode. Use "False" if you don't need this.

* pidfile - REQUIRED - Path to a pidfile. If #autopid# is False, your app must write its pid to this file. Otherwise, usrsvcd will mangage it, even with #scan_for_process# or other methods.

* enabled - Boolean, default True. Set to "False" to disable the program from being managed by "usrsvcd"

* autopid - Default True, boolean. If True, "usrsvc" and "usrsvcd" will write the pid of the launched program to the pidfile, i.e. managed. If your application forks-and-exits, you can set this to FAlse and write your own pid, or use #scan_for_process#

* scan_for_process - Default True, boolean. If True, "usrsvc" and "usrsvcd" will, in the absense of a pidfile which matches with #proctitle_re#, use #proctitle_re# and scan running processes for the application. This can find applications even when the pidfile has gone missing.

* proctitle_re - None or a regular expression which will match the proctitle (can be seen as last col in "ps auxww").  If none provided, a default wherein the command and arguments are used, will work in almost all instances. Some applications modify their proctitle, and you may need to use this to match them.



* autostart - Default True, boolean value if program should be started if not already running when "usrsvcd" is invoked

* autorestart - Default True, boolean value if program should be restarted if it stopped while "usrsvcd" is running

* maxrestarts - Default 0, integer on the max number of times usrsvcd will try to automatically restart the application by "usrsvcd". If it is seen running again naturally, this counter will reset. 0 means unlimited restarts.

* restart_delay - Default 0, integer on the miminum number of seconds between a failing "start" and the next "restart" attmept by "usrsvcd". 

* success_seconds - Default 2, Float on the number of seconds the application must be running for "usrsvc" to consider it successfully started.

* term_to_kill_seconds : Default 8, Float on the number of seconds the application is given between SIGTERM and SIGKILL.



NOTE: The following stdout/stderr are opened in "append" mode always. 

* stdout - REQUIRED - Absolute path to a file to be used for stdout

* stderr - Absolute path to a file to be used for stderr, or "stdout" to redirect to stdout. Default is to redirect stderr to stdout. May be same filename as stdout.

* defaults - This can reference a "DefaultSettings" section defined elsewhere, i.e. to reference [DefaultSettings:MySettings] use "defaults=MySettings". If provided, this Program will inherit the settings defined in the DefaultSettings as the defaults. Anything provided explicitly in this Program will override those found in the defaults.

* inherit_env - Boolean, default True. If True, will inherit the env from "usrsvc" or "usrsvcd". Otherwise, will only use the Env as defined in the Env subsection.



"Program" Supports the following subsections:

[[Monitor]]

The monitoring section, see below for more info.

[[Env]]

A series of key=value items which will be present in the environment prior to starting this Program.


*Example Program Config:* 


	[Program:myprogram]


	command = /home/myusr/bin/myprogram

	pidfile = /home/myusr/pids/myprogram.pid

	stdout  = /home/myusr/logs/myprogram.log

	stderr  = stdout


	[[Env]]


		DB_USER = superdb

		DB_NAME = mydatabase



**[DefaultSettings:mydefaults]**

These define a set of default settings for a Program, and can include default values in subsections as well. Your program can inherit these default settings by setting the "defaults=mydefaults" property, where "mydefaults" is the name of your DefaultSettings.


**[[Monitor]]**

The Monitor subsection specifies if and how your Program will be monitored. This is to sense if your Program has frozen and needs a restart, the "autorestart" and "autostart" monitoring are handled in the "Program" config.

Note, additional Monitoring types will be available in a future release.

Monitor can contain the following properties:

* monitor_after - Minimum number of seconds that program needs to be running before monitoring will begin. Default 30. 0 disables this feature.


(Activity File Monitoring)

The following two properties deal with "activity file" monitoring, that is ensuring that a file or directory is updated within a specified number of seconds.


* activityfile - File or Directory which must be modified every #activityfile_limit# seconds, or program will be restarted. Default undefined/empty string disables this.

* activityfile_limit - If activityfile is defined, this is the number of seconds is the maximum that can go between modifications of the provided #activityfile# before triggering a restart.


(RSS Limit Monitoring)


The following property triggers the "rss limit" monitor. This monitor checks the Resident Set Size (non-shared memory an application is using), and restarts if it exceeds a given threshold.

* rss_limit - Default 0, if greater than zero, specifies the maximum RSS (resident set size) that a process may use before being restarted. This is the "private" memory (not including shared maps, etc) used by a process.

Examples
--------

An example configuration can be found in the "examples" directory ( https://github.com/kata198/usrsvc/tree/master/examples ). The "usrsvc.cfg" is the main configuration file (to be located in $HOME/usrsvc.cfg), and the "cfg" directory is intended to be "/home/myusr/usrsvc.d/cfg" (per config_dir value in usrsvc.cfg

Contact Me
----------

You may reach me for support, questions, feature requests, or just to let me know you're using it! Use the email kata198 at gmail.

Changes
-------

The Changelog can be found at: https://raw.githubusercontent.com/kata198/usrsvc/master/ChangeLog

"""



'''
    Copyright (c) 2016 Tim Savannah All Rights Reserved.
    This software is licensed under the terms of the GPLv3.
    This may change at my discretion, retroactively, and without notice.

    You should have received a copy of this with the source distribution as a file titled, LICENSE.

    The most current license can be found at:
    https://github.com/kata198/usrsvc/LICENSE

    This location may need to be changed at some point in the future, in which case
    you are may email Tim Savannah <kata198 at gmail dot com>, or find them on the
    current website intended for distribution of usrsvc.



    'UsrsvcConfig' is the main config class used within usrsvc
'''

import copy
import os
import glob

from configobj import ConfigObj

from .MainConfig import MainConfig
from .ProgramConfig import ProgramConfig

__all__ = ('UsrsvcConfig', )

class UsrsvcConfig(object):
    '''
        UsrsvcConfig - The main config class. All other configs are accessable through this.
    '''

    def __init__(self, mainConfigFile):
        self.mainConfigFile = mainConfigFile

        self.isPared = False
        self.datas = None

        self.mainConfig = None
        self.programConfigs = {}

    def parse(self):
        '''
            parse - Parse the config data.
        '''
        if not os.path.exists(self.mainConfigFile):
            raise ValueError('File does not exist: %s' %(self.mainConfigFile,))

        self.datas = []
        self.filenames = [self.mainConfigFile]

        # Process main config
        mainData = ConfigObj(self.mainConfigFile)
        self.datas.append(mainData)

        if 'Main' not in mainData:
            raise ValueError('Missing [Main] section in main config. Required.')
        
        self.mainConfig = MainConfig(**mainData['Main'])

        self.defaultSettings = {}

        # Get additional configs, read all data.
        programConfigDir =  self.mainConfig.getProgramConfigDir()
        if programConfigDir:
            programConfigFiles = glob.glob(programConfigDir + '/*.cfg')
            for fname in programConfigFiles:
                data = ConfigObj(fname)
                self.datas.append(data)
                self.filenames.append(fname)

        # Run through all configs to gather DefaultSettings
        for i in range(len(self.datas)):
            data = self.datas[i]
            fname = self.filenames[i]
            for key in data.keys():
                if key.startswith('DefaultSettings:'):
                    item = data[key]
                    name = key[len('DefaultSettings:'):]
                    if not name:
                        raise ValueError('DefaultSettings section defined without a name in "%s"' %(fname,))
                    if name in self.defaultSettings:
                        raise KeyError('Multiple DefaultSettings object with name "%s". Second encountered in "%s"' %(name, fname))

                    self.defaultSettings[name] = item

        for i in range(len(self.datas)):
            data = self.datas[i]
            fname = self.filenames[i]
            for key in data.keys():
                if key.startswith('Program:'):
                    item = data[key]
                    name = key[len('Program:'):]
                    if 'defaults' in item:
                        defaultName = item['defaults']
                        if defaultName not in self.defaultSettings:
                            raise ValueError('Program "%s" in file "%s" uses a "defaults" of "%s", but no such DefaultSettings section exists in read configuration files!' %( name, fname, defaultName))
                        item2 = self.defaultSettings[defaultName]
                        item2 = copy.deepcopy(item2)
                        # Merge subsections if present
                        if 'Env' in item2 and 'Env' in item:
                            item2['Env'].update(item['Env'])
                            item['Env'] = item2['Env']
                            del item2['Env']
                        if 'Monitoring' in item2 and 'Monitoring' in item:
                            item2['Monitoring'].update(item['Monitoring'])
                            item['Monitoring'] = item2['Monitoring']
                            del item2['Monitoring']

                        item2.update(item)
                        item = item2

                    self.programConfigs[name] = ProgramConfig(name, **item)



    def getProgramConfig(self, programName):
        '''
            getProgramConfig - get the ProgramConfig associated with a program.

            @param programName <str> - Name of program

            @raises KeyError if program not found

            @return ProgramConfig of the program
        '''
        return self.programConfigs[programName]

    def getProgramConfigs(self):
        '''
            getProgramConfigs - Get all program configs.

            @return <dict> - program name <str> to ProgramConfig
        '''
        return self.programConfigs



'''
    Copyright (c) 2016 Tim Savannah All Rights Reserved.
    This software is licensed under the terms of the GPLv3.
    This may change at my discretion, retroactively, and without notice.

    You should have received a copy of this with the source distribution as a file titled, LICENSE.

    The most current license can be found at:
    https://github.com/kata198/usrsvc/LICENSE

    This location may need to be changed at some point in the future, in which case
    you are may email Tim Savannah <kata198 at gmail dot com>, or find them on the
    current website intended for distribution of usrsvc.


    ProgramActions - Some common actions dealing with process management.
'''

from .Program import Program
from .logging import logErr, logMsg

# TODO: Notification system

def getRunningProgram(programConfig):
    '''
        getRunningProgram - Get a "Program" object representing a running program, or None if not running.

            Also handles validating proctitle and removing stale pid files.

            Tries pidfile first, if no match then if scan_for_process is set on the programConfig, running processes will
              be scanned and if a proctitle match is found, a pidfile wiil be written and the program returned.

         @param programConfig <ProgramConfig obj> - The ProgramConfig object representing the program to fetch.

         @return <None/Program> - A running Program if match was found, otherwise None.
    '''
    programName = programConfig.name

    prog = None
    try:
        # Try to get program from pidfile
        prog = Program.createFromPidFile(programConfig.pidfile)
    except:
        pass

    if prog is not None:
        # A program is running, check if it is the RIGHT program
        isCorrectApp = prog.validateProcTitle(programConfig)
        if not isCorrectApp:
            logErr('Warning: Detected stale pid file for %s at %s (%d). REMOVING\n  Proctitle for %d was: %s\n' %(programName, programConfig.pidfile, prog.pid, prog.pid, prog.cmdline))
            prog.removePidFile(programConfig)
            prog = None
        else:
            # Program is running, and matched proctitle
            return prog
            
    if prog is None:
        if programConfig.scan_for_process is True:
            # No PID or failed proctitle match, scan running processes
            prog = Program.createFromRunningProcesses(programConfig)
            if prog:
                logMsg('Matched %s from running process:\n\n%s\n' %(programName, prog.__dict__))
                prog.writePidFile(programConfig)
                return prog

    return prog



'''
    Copyright (c) 2016 Tim Savannah All Rights Reserved.
    This software is licensed under the terms of the GPLv3.
    This may change at my discretion, retroactively, and without notice.

    You should have received a copy of this with the source distribution as a file titled, LICENSE.

    The most current license can be found at:
    https://github.com/kata198/usrsvc/LICENSE

    This location may need to be changed at some point in the future, in which case
    you are may email Tim Savannah <kata198 at gmail dot com>, or find them on the
    current website intended for distribution of usrsvc.


    'MainConfig' is the main configuration file
'''

import os

__all__ = ('MainConfig', )

class MainConfig(object):
    '''
        MainConfig - The main config object. Will read all child configs, and provides access to all configs through this object.

        This object provides a refreshable means to update runtime configuration. 
        The main op iterations should fetch the relevant sections, and on next loop fetch from new.
    '''

    def __init__(self, config_dir=None, pidfile=None, usrsvcd_stdout=None, usrsvcd_stderr=None, **kwargs):
        if kwargs:
            raise ValueError('Unknown config options in Main section: %s\n' %(str(list(kwargs.keys())),))

        if config_dir:
            if config_dir[0] != '/':
                raise ValueError('config_dir in [Main], if defined, must be an absolute path.')
            if config_dir[-1] == '/':
                config_dir = config_dir[:-1]

        self.config_dir = config_dir
        self.pidfile = pidfile or (os.environ.get('HOME', '/tmp') + '/%d_usrsvcd.pid' %(os.getuid()))
        if usrsvcd_stdout:
            if usrsvcd_stdout[0] != '/':
                raise ValueError('usrsvcd_stdout in [Main], if defined, must be an absolute path.')

        self.usrsvcd_stdout = usrsvcd_stdout

        if usrsvcd_stderr:
            if usrsvcd_stderr != 'stdout':
                if usrsvcd_stderr[0] != '/':
                    raise ValueError('usrsvcd_stderr in [Main], if defined, must be "stdout" or an absolute path.')

        self.usrsvcd_stderr = usrsvcd_stderr

    def getProgramConfigDir(self):
        return self.config_dir

    def __str__(self):
        return str(self.__dict__)



'''
    Copyright (c) 2016 Tim Savannah All Rights Reserved.
    This software is licensed under the terms of the GPLv3.
    This may change at my discretion, retroactively, and without notice.

    You should have received a copy of this with the source distribution as a file titled, LICENSE.

    The most current license can be found at:
    https://github.com/kata198/usrsvc/LICENSE

    This location may need to be changed at some point in the future, in which case
    you are may email Tim Savannah <kata198 at gmail dot com>, or find them on the
    current website intended for distribution of usrsvc.


    usrsvc is a user process manager
'''

import time

__all__ =  ('waitUpTo', )


def waitUpTo(testFunc, funcArgs, timeout, interval=0.1):
    '''
        waitUpTo - Wait up to a certain amount of time, or until a provided test function
            returns False.

        @param testFunc <function/lambda> - Test function, True if keep going else False
        @param funcArgs <list/tuple> - Args to past to test function
        @param timeout <float> - Max number  of seconds to wait
        @param interval <float> - Interval between polls. Does not make sense to be > timeout.

        @return - True if timeout occured (function never returned False), otherwise False.
    '''
    if not isinstance(funcArgs, tuple):
        funcArgs = tuple(funcArgs)

    maxTime = time.time() + timeout
    keepGoing = funcResult = testFunc(*funcArgs)

    while keepGoing is True:
        time.sleep(interval)
        now = time.time()
        funcResult = testFunc(*funcArgs)
        keepGoing = (now < maxTime and funcResult is True)

    return not funcResult




'''
    Copyright (c) 2016 Tim Savannah All Rights Reserved.
    This software is licensed under the terms of the GPLv3.
    This may change at my discretion, retroactively, and without notice.

    You should have received a copy of this with the source distribution as a file titled, LICENSE.

    The most current license can be found at:
    https://github.com/kata198/usrsvc/LICENSE

    This location may need to be changed at some point in the future, in which case
    you are may email Tim Savannah <kata198 at gmail dot com>, or find them on the
    current website intended for distribution of usrsvc.

    usrsvc is the tool to start/stop/restart and get status on services. usrsvcd is the daemon for autostart, autorestart, monitoring, etc.
'''


import os
import copy
import signal
import multiprocessing
import sys
import traceback

from NamedAtomicLock import NamedAtomicLock

from usrsvcmod.UsrsvcConfig import UsrsvcConfig
from usrsvcmod.Program import Program
from usrsvcmod.ProgramActions import getRunningProgram
from usrsvcmod.logging import logMsg, logErr
from usrsvcmod.constants import ReturnCodes

class Usrsvc(object):

    def __init__(self, config=None):
        if not config:
            config = UsrsvcConfig(os.environ['HOME'] + '/usrsvc.cfg')
            try:
                config.parse()
            except ValueError as e:
                sys.stderr.write('ERROR in config: %s\n'  %(str(e),))
                raise e

        self.config = config

    def call(self, args):
        argv = args[:]
        if argv[0] != 'usrsvc':
            argv = ['usrsvc'] + args

        process = multiprocessing.Process(target=self._call_main, args=(argv,))
        process.start()
        process.join()

        return process

    def _call_main(self, argv):
        sys.exit(self.main(argv))

    def doActionParallel(self, action):
        config = self.config

        allProgramNames = config.getProgramConfigs().keys()
        processes = {}
        for programName in allProgramNames:
            process = multiprocessing.Process(target=self.doActionAndExit, args=([action, programName], ))
            process.start()
            processes[programName] = process

        ret = ReturnCodes.SUCCESS
        exitCodes = {}
        for processName, process in processes.items():
            process.join()
            exitCodes[processName] = process.exitcode

        for programName, exitCode in exitCodes.items():
            if exitCode != ReturnCodes.SUCCESS:
                ret = ReturnCodes.GENERAL_FAILURE
                logErr('%s exited with non-zero return code: %d (%s)\n' %(programName, exitCode, ReturnCodes.returnCodeToString(exitCode)) )

        return ret

    def doAction(self, args):
        config = self.config

        action = args[0]
        if args[1] == 'all':
            # Serial start, parallel is handled elsewhere
            ret = 0
            for programName in config.getProgramConfigs().keys():
                try:
                    exitCode = self.doAction([action, programName])
                except Exception as e:
                    logErr('Unexpected exception (%s) trying to %s %s: %s\n' %(str(e.__class__.__name__), action, programName, str(e)))
                    traceback.print_exc()
                    exitCode = ReturnCodes.UNKNOWN_FAILURE
                if exitCode != 0:
                    logErr('%s failed to %s with error %d (%s)\n' %(programName, action, exitCode, ReturnCodes.returnCodeToString(exitCode)) )
                    ret = ReturnCodes.GENERAL_FAILURE
            return ret
        
        programName = args[1]

        lock = NamedAtomicLock('.lock_usrsvc' + programName, maxLockAge=30)
        if not lock.acquire(31):
            logErr('Cannot acquire lock for %s. Is something else looping trying to access it? Try the command again.\n' %(programName,))
            return ReturnCodes.TRY_AGAIN
        ret = self._doAction(args)
        lock.release()
        return ret

    def _doAction(self, args):
        config = self.config

        action = args[0]
        programName = args[1]

        try:
            programConfig = config.getProgramConfig(programName)
        except KeyError:
            logErr('No such program: %s\n' %(programName,))
            return ReturnCodes.PROGRAM_UNDEFINED

        if programConfig.enabled is False and action != 'status':
            logErr('Program %s is currently disabled in config. Only the "status" action is supported on disabled programs.\n' %(programName,))
            return ReturnCodes.PROGRAM_DISABLED

        if action == 'start':
            prog = None
            try:
                prog = Program.createFromPidFile(programConfig.pidfile)
            except:
                pass

            prog = getRunningProgram(programConfig)
            if prog is not None:
                logMsg('Program %s is already running:\n\n%s\n' %(programName, prog.__dict__))
                return ReturnCodes.SUCCESS

            prog = Program(programConfig.pidfile)
            returnCode = prog.startProgram(programConfig)
            
            if returnCode == ReturnCodes.SUCCESS:
                logMsg('Started %s:\n\n%s\n' %(programName, prog.__dict__))
                return ReturnCodes.SUCCESS
            else:
                logErr('Failed to start %s! Err=%d (%s)\n' %(programName, returnCode, ReturnCodes.returnCodeToString(returnCode)))
                return returnCode

        elif action == 'stop':
            
            prog = getRunningProgram(programConfig)
            if prog:
                logMsg('Stopping %s [%d]\n' %(programName, prog.pid))
                actionTaken = prog.stopProgram(programConfig)
                logMsg('%s %s\n' %(programName, actionTaken))
            else:
                logMsg('%s was not running.\n' %(programName,))
            return ReturnCodes.SUCCESS
        elif action == 'restart':
            self._doAction(['stop'] + args[1:])
            return self._doAction(['start'] + args[1:])
        elif action == 'status':
            prog = getRunningProgram(programConfig)
            if prog:
                logMsg('%s is running:\n\n%s\n' %(programName, str(prog.__dict__)))
                return ReturnCodes.SUCCESS
            else:
                logErr('%s is NOT running\n' %(programName,))
                return ReturnCodes.GENERAL_FAILURE
        else:
            logErr('Unknown action: %s\n' %(action,))
            return ReturnCodes.INVALID_ACTION
     

    def doActionAndExit(self, args):
        sys.exit(self.doAction(args))


    @classmethod
    def printUsage(cls):
        sys.stderr.write('''Usage: usrsvc (Options) [start/stop/restart/status] [program name]
 Performs the requested action on the given program name.
 "all" can be used for start/stop/restart in place of "program name"
 
usrsvc is tool for performing specific actions on services, usrsvcd is the related daemon for autorestart/monitoring, etc.

  Options:
  --------

    Parallel:
        When doing start/stop/restart all, you may add "--parallel" to perform 
        the action on all items in parallel.
          

  Config:
  -------

    Usrsvc uses the config file found at $HOME/usrsvc.cfg (%s).

  Documentation
  -------------

    Run "usrsvc --readme" or see https://github.com/kata198/usrsvc/blob/master/README.md 
      for more documentation.

''' %(os.environ['HOME'] + '/usrsvc.cfg', ) )

    
    def main(self, argv):
        parallelAll = False
        try:

            if '--help' in argv:
                self.printUsage()
                return ReturnCodes.HELP_MESSAGE
            elif '--readme' in argv:
                from usrsvcmod.client.readme import printReadme
                printReadme(sys.stdout)
                return ReturnCodes.HELP_MESSAGE
            elif len(argv) == 4 and argv[1] in ('start', 'stop', 'restart') and argv[2] == 'all' and argv[3] == '--parallel':
                parallelAll = True
            elif len(argv) != 3:
                logErr('Invalid number of arguments.\n')
                self.printUsage()
                return ReturnCodes.GENERAL_FAILURE

            # Prevent signals from interrupting us
            signal.signal(signal.SIGTERM, signal.SIG_IGN)
            signal.signal(signal.SIGINT, signal.SIG_IGN)

            if parallelAll is True:
                return self.doActionParallel(argv[1])
            else:
                return self.doAction(argv[1:])

        except Exception  as  e:
           logErr('Error  in main %s:  %s\n'  %(str(argv), str(e)))
           return ReturnCodes.UNKNOWN_FAILURE





'''
    Copyright (c) 2016 Tim Savannah All Rights Reserved.
    This software is licensed under the terms of the GPLv3.
    This may change at my discretion, retroactively, and without notice.

    You should have received a copy of this with the source distribution as a file titled, LICENSE.

    The most current license can be found at:
    https://github.com/kata198/usrsvc/LICENSE

    This location may need to be changed at some point in the future, in which case
    you are may email Tim Savannah <kata198 at gmail dot com>, or find them on the
    current website intended for distribution of usrsvc.


    'Program' represents a program, running or not.
'''

import os
import signal
import shlex
import subprocess
import time

from .constants import ReturnCodes
from .logging import logMsg, logErr
from .util import  waitUpTo

__all__ = ('Program',)

class Program(object):
    '''
        Program - defines a program, running or not.
    '''


    def __init__(self, pidfile=None, pid=None, cmdline=None, executable=None, args=None, running=False):
        '''
            Create a program.

            @param pidfile - <str/None> - path to pidfile
            @param pid - <int/None> - pid of program
            @param cmdline - <str/None> - cmdline of program
            @param executable - <str/None> - executable or None
            @param args - <list/None> - List of args, or None.
            @param running - <bool> - True/False.

            Should not be created direcly, use one of the static methods to create this.
        '''
        self.pid = pid or None
        # Note: careful with the cmdline/executable/args, maybe not all being set. Maybe need to ensure with a helper method.
        self.cmdline = cmdline or None
        self.executable = executable or None
        self.args = args or []
        self.running = running

        
    @staticmethod
    def _getProcCmdline(pid):
        '''
            _getProcCmdline - Gets the /proc/$pid/cmdline , split into string, executable, and args, for a given pid.

            @return - dict {
                cmdline - string of commandline
                executable - string of executable
                args - list of arguments
                }

        '''
        with open('/proc/%d/cmdline' %(pid,), 'rb') as f:
            cmdlineContents = f.read()
        
        # Split by null, and strip the trailing null
        cmdlineSplit = cmdlineContents.split(b'\x00')[:-1]
        executable = cmdlineSplit[0].decode('utf-8')
        args = [x.decode('utf-8') for x in cmdlineSplit[1:]]

        cmdline = ' '.join([executable] + args)

        return {
            'cmdline' : cmdline,
            'executable' : executable,
            'args' : args
        }

    def setCmdlineFromProc(self, forcePid=None):
        '''
            setCmdlineFromProc - Sets the commandline properties (cmdline, exectuable, args) from the proc value.

            @param forcePid - Uses the pid on this object as default, but you can force another with this param.

        '''
        if forcePid is None:
            forcePid = self.pid
        if not forcePid:
            raise ValueError('Cannot call setCmdlineFromProc on a program without an associated pid.')

        cmdlineInfo = self._getProcCmdline(forcePid)

        self.cmdline = cmdlineInfo['cmdline']
        self.executable = cmdlineInfo['executable']
        self.args = cmdlineInfo['args']

    def validateProcTitle(self, programConfig):
        '''
            validateProcTitle - uses proctitle_re to validate if the proctitle associated with the pid associated with this object
              matches.

            @return -<bool> If the pid associated with this Program has a proctitle that matches
        '''
        if not programConfig.proctitle_re:
            # Disabled proctitle matching somehow. So always match. BAD IDEA!
            return True

        return bool(programConfig.proctitle_re.search(self.cmdline))

    @classmethod
    def createFromPidFile(cls, pidfile):
        '''
            createFromPidFile - Creates a Program from a pidfile.

            @param pidfile <str> - Path to pid file

            @raises - An exception if no process can be created from the pidfile. Specific exception depends on error. Not validated with proctitle_re, do that in an additional call.
        '''
        with open(pidfile, 'rt') as f:
            pid = f.read().strip()
        pid = int(pid)
        procCmdline = cls._getProcCmdline(pid)

        return cls(pidfile, pid, running=True, **procCmdline)

    @classmethod
    def createFromRunningProcesses(cls, programConfig):
        '''
            createFromRunningProcesses - Create a Program using programConfig. scans all running processes, and returns a constructed Program if a match is found.

            @param programConfig <ProgramConfig.ProgramConfig> - The config for a program

            @return - A constructed "Program" if a match found, ootherwise None.

            Should not raise anything.
        '''
        myUid = os.getuid()
        allMyPids = []
        # Do in a loop incase processes are created/destroyed between getting the list and checking it
        for procItem in os.listdir('/proc'):
            try:
                if procItem.isdigit() and os.stat('/proc/' + procItem).st_uid == myUid:
                    allMyPids.append(int(procItem))
            except:
                pass

        proctitleRE = programConfig.proctitle_re

        for pid in allMyPids:
            try:
                # Do in a loop incase processes are created/destroyed between getting the list and checking it
                procCmdline = cls._getProcCmdline(pid)
                if bool(proctitleRE.search(procCmdline['cmdline'])):
                    prog = cls(programConfig.pidfile, pid, running=True, **procCmdline)
                    return prog
            except:
                pass

        return None
                    

    @classmethod
    def isProgramRunning(cls, programConfig):
        '''
            isProgramRunning - Actively check if a program is running

            @param programConfig <ProgramConfig.ProgramConfig> - The config of a program

            @return - True if the program is running and matches the proctitle.

            NOT USED!

            Use ProgramActions.getRunningProgram instead. That will return the program itself, and manage the pidfile.

        '''
        prog = None
        try:
            prog = cls.createFromPidFile(programConfig.pidfile)
        except:
            pass

        if prog is None and programConfig.scan_for_process is True:
            try:
                prog = Program.createFromRunningProcesses(programConfig)
                return True # Don't need to re-validate the proc title
            except:
                return False

        if prog is None:
            return False

        return prog.validateProcTitle(programConfig)


    def getStartTime(self):
        '''
            getStartTime - Get the epoch timestamp for when this process was started.

            @return - time process was started, or None if no program found matching this Program's pid.
        '''
        try:
            statInfo = os.stat('/proc/' + str(self.pid))
            if not statInfo:
                return None
            # Try ctime first, but may not be present everywhere, so fallback to st_mtime
            if hasattr(statInfo, 'st_ctime') and statInfo.st_ctime:
                return statInfo.st_ctime
            return statInfo.st_mtime
        except Exception:
            pass

        return None


    def startProgram(self, programConfig):
        '''
            startProgram - Start the program and update this object.

            @param programConfig <ProgramConfig.ProgramConfig> - The program config associated with this program.
        '''
        useShell = programConfig.useshell

        if useShell is False:
            command = shlex.split(programConfig.command)
        else:
            command = programConfig.command

        try:
            stdout = open(programConfig.stdout, 'at')
        except Exception as e:
            logErr('(%s) - Cannot open stdout %s for writing: %s\n' %(programConfig.name, programConfig.stdout, str(e)))
            return ReturnCodes.INSUFFICIENT_PERMISSIONS
#            raise ValueError('Cannot open %s for writing.' %(programConfig.stdout,))
        if programConfig.stdout == programConfig.stderr:
            stderr = stdout
        else:
            try:
                stderr = open(programConfig.stderr, 'at')
            except Exception as e:
                logErr('(%s) - Cannot open stderr %s for writing: %s\n' %(programConfig.name, programConfig.stderr, str(e)))
                return ReturnCodes.INSUFFICIENT_PERMISSIONS
#                raise ValueError('Cannot open %s for writing.' %(programConfig.stderr,))

        if programConfig.inherit_env:
            env = os.environ
        else:
            env = {}

        if hasattr(subprocess, 'DEVNULL'):
            devnull = subprocess.DEVNULL
        else:
            devnull = open('/dev/null', 'rt')

        env.update(programConfig.Env)

        try:
            pipe = subprocess.Popen(command, shell=useShell, stdout=stdout, stderr=stderr, stdin=devnull, env=env, close_fds=False)
        except Exception as e:
            logErr('(%s) - Failed to run command ( %s ): %s\n' %(programConfig.name, str(command), str(e)))
            return ReturnCodes.PROGRAM_FAILED_TO_LAUNCH
        
        now = time.time()
        successAfter = now + programConfig.success_seconds

        success = True

        pollTime = min(programConfig.success_seconds, .1)
        while time.time() < successAfter:
            time.sleep(pollTime)
            pollResult = pipe.poll()
            if pollResult is not None:
                logErr('(%s) - Exited with code=%d\n' %(programConfig.name, pollResult))
                success = False
                break

        if success is False:
            # We've failed to restart after the given number of tries.
            return ReturnCodes.PROGRAM_EXITED_UNEXPECTEDLY

        # Started successfully, update our dict.
        self.pid = int(pipe.pid)
        try:
            self.setCmdlineFromProc()
        except:
            # Oh no! By some fluke, the program stopped running while we were setting it up. Return False....
            self.running = False
            return ReturnCodes.PROGRAM_EXITED_UNEXPECTEDLY

        self.running = True

        self.writePidFile(programConfig, self.pid)

        return ReturnCodes.SUCCESS


    def stopProgram(self, programConfig):
        '''
            stopProgram - Stop this Program and remove pid file.

            @param programConfig <ProgramConfig.ProgramConfig> - The ProgramConfig for this program

            @return <str> - Returns a string of what happened. Can be one of "noaction", "terminated" (SIGTERM), "killed" (SIGKILL)

        '''

        result = 'no action'

        if self.pid and self.validateProcTitle(programConfig):
            try:
                os.kill(self.pid, signal.SIGTERM)
                result = 'terminated'

                termToKillSeconds = programConfig.term_to_kill_seconds

                pollInterval = min(termToKillSeconds / 10.0, .1)
                processDied = waitUpTo(os.path.exists, ('/proc/%d' %(self.pid,) , ), timeout=termToKillSeconds, interval=pollInterval)
                if processDied is False:
                    # If program has not terminated given the threshold, send 'er the ol' boot.
                    os.kill(self.pid, signal.SIGKILL)
                    result = 'killed'

            except:
                pass

        # Remove pid file
        self.removePidFile(programConfig)

        return result


    def writePidFile(self, programConfig, forcePid=None):
        '''
            writePidFile - write the pid file for this program.

            @param programConfig <ProgramConfig.ProgramConfig> - The ProgramConfig for this program
            
            @param forcePid <int/None> - if provided, use this pid instead of self.pid

            @return - False if not written (no autopid), otherwise True if written.
        '''
        if forcePid:
            pid = forcePid
        else:
            pid = self.pid

        # Note: pidfile should never be undefined... but just incase somehow.
        if programConfig.autopid is False or not programConfig.pidfile:
            return False
        with open(programConfig.pidfile, 'wt') as f:
            f.write("%s\n" %(str(pid),))
        return True
        

    def removePidFile(self, programConfig):
        '''
            removePidFile - Removes the pid file associated with this Program.

            @param programConfig <ProgramConfig.ProgramConfig> - The ProgramConfig for this program
        '''

        try:
            if os.path.exists(programConfig.pidfile):
                os.unlink(programConfig.pidfile)
            return True
        except Exception as e:
            if os.path.exists(programConfig.pidfile):
                # Check again incase something else removed it, we only want to alert when we couldn't remove it.
                logErr('(%s) Unable to remove pidfile %s: %s\n' %(programConfig.name, programConfig.pidfile, str(e)))
            return False

    def __str__(self):
        return str(self.__dict__)



'''
    Copyright (c) 2016 Tim Savannah All Rights Reserved.
    This software is licensed under the terms of the GPLv3.
    This may change at my discretion, retroactively, and without notice.

    You should have received a copy of this with the source distribution as a file titled, LICENSE.

    The most current license can be found at:
    https://github.com/kata198/usrsvc/LICENSE

    This location may need to be changed at some point in the future, in which case
    you are may email Tim Savannah <kata198 at gmail dot com>, or find them on the
    current website intended for distribution of usrsvc.


    usrsvc is a user process manager
'''

__version__ = '1.3.0'
__version_tuple__ = (1, 3, 0)



'''
    Copyright (c) 2016 Tim Savannah All Rights Reserved.
    This software is licensed under the terms of the GPLv3.
    This may change at my discretion, retroactively, and without notice.

    You should have received a copy of this with the source distribution as a file titled, LICENSE.

    The most current license can be found at:
    https://github.com/kata198/usrsvc/LICENSE

    This location may need to be changed at some point in the future, in which case
    you are may email Tim Savannah <kata198 at gmail dot com>, or find them on the
    current website intended for distribution of usrsvc.


    usrsvc is a user process manager
'''
import re

__all__ = ('getConfigValueBool', 'getConfigValueInt', 'getConfigValueFloat')

# Following functions all name the value, and the config name.

def getConfigValueBool(value, name=''):
    if type(value) == bool:
        return value
    else:
        value = str(value)
        if value.isdigit():
            return bool(int(value))
        value = value.lower()
        if value == 'true':
            return True
        if value == 'false':
            return False

    if name:
        errPart = ' for config parameter %s' %(name,)
    else:
        errPart = ''

    raise ValueError('Unknown value%s: %s. Should be True/1 or False/0' %(errPart, value) )

# Keep in mind that getConfigValueInt and getConfigValueFloat assume positive values.
#   Will need to be modified if we ever need to support a negative config value.

def getConfigValueInt(value, name=''):
    if type(value) == int:
        return value
    elif type(value) == float:
        return int(value)
    else:
        if not value.isdigit():
            raise ValueError('Invalid value for config parameter %s. Should be a positive integer. Got %s' %(name, str(value)))
        return int(value)



FLOAT_RE = re.compile('^(?P<int_part>[\d]*)([\.](?P<decimal_part>[\d]+)){0,1}$')

def getConfigValueFloat(value, name=''):
    if type(value) == float:
        return value
    elif type(value) == int:
        return float(value)
    else:
        if not FLOAT_RE.match(value):
            raise ValueError('Invalid value for config parameter %s. Should be a positive floating-point number. Got %s' %(name, str(value)))
        return float(value)


        



import sys

try:
    from usrsvcmod.client._readme_contents import README_CONTENTS
except ImportError:
    README_CONTENTS = "Extended help not available -- usrsvc was not installed correctly.\n"

def printReadme(outObj=None):
    if outObj is None:
        outObj = sys.stdout
    
    sys.stdout.write(README_CONTENTS)
    sys.stdout.flush()



'''
    Copyright (c) 2016 Tim Savannah All Rights Reserved.
    This software is licensed under the terms of the GPLv3.
    This may change at my discretion, retroactively, and without notice.

    You should have received a copy of this with the source distribution as a file titled, LICENSE.

    The most current license can be found at:
    https://github.com/kata198/usrsvc/LICENSE

    This location may need to be changed at some point in the future, in which case
    you are may email Tim Savannah <kata198 at gmail dot com>, or find them on the
    current website intended for distribution of usrsvc.


    ActivityFileMonitor - Asserts that a specific file or directory should be modified within a certain threshold
'''

import os
import time

from func_timeout import FunctionTimedOut

from . import MonitoringBase
from ..logging import logMsg, logErr

# TODO: We need to implement the check here as launching and joining on a thread, so that we don't lockup all monitoring if someone
#  uses an NFS file on a disconnected device or anything else that will result in an indefinite uninterruptable ("D") state.

class ActivityFileMonitor(MonitoringBase):
    '''
        ActivityFileMonitor - Class for doing activity file monitoring
    '''

    def __init__(self, programName, activityFile, activityFileLimit):
        self.programName = programName
        self.activityFile = activityFile
        self.activityFileLimit = activityFileLimit

    @classmethod
    def createFromConfig(cls, programConfig):
        if not programConfig.Monitoring.activityfile:
            return None

        return cls(programConfig.name, programConfig.Monitoring.activityfile, programConfig.Monitoring.activityfile_limit)

    def shouldRestart(self, program=None):
        '''
            Returns True if activity file has not been modified within the threshold specified by activityfile_limit (should restart), otherwise False.

            @param program - unused.
        '''
        activityFile = self.activityFile
        activityFileLimit = self.activityFileLimit
        programName = self.programName

        if not activityFile:
            # Yes this is checked twice if created through createFromConfig, but it may be called otherwise so better safe.
            return False

        try:
            # If activity file is not present, this is a fail and we restart.
            if not os.path.exists(activityFile):
                logMsg('Restarting %s because activity file ( %s ) does not exist\n' %(programName, activityFile,))
                return True
            # Gather the mtime and see if we are past the threshold
            lastModified = os.stat(activityFile).st_mtime
            threshold = float(time.time() - self.activityFileLimit)
            if lastModified < threshold:
                logMsg('Restarting %s because it has not modified activity file ( %s ) in %.4f seconds. Limit is %d.\n' %(programName, activityFile, float(threshold - lastModified), activityFileLimit))
                return True
        except FunctionTimedOut:
            logErr('MONITOR: ActivityFile timed out on %s\n' %(programName,))
            raise

        except Exception as e:
            # If we got an exception, just log and try again next round.
            logErr('Got an exception in activity file monitoring. Not restarting program. Program="%s" activityfile="%s"\nlocals: %s\n' %(programName, activityFile, str(locals())))

        return False 





'''
    Copyright (c) 2016 Tim Savannah All Rights Reserved.
    This software is licensed under the terms of the GPLv3.
    This may change at my discretion, retroactively, and without notice.

    You should have received a copy of this with the source distribution as a file titled, LICENSE.

    The most current license can be found at:
    https://github.com/kata198/usrsvc/LICENSE

    This location may need to be changed at some point in the future, in which case
    you are may email Tim Savannah <kata198 at gmail dot com>, or find them on the
    current website intended for distribution of usrsvc.


    Kind of a psuedo-factory pattern for Monitoring. Not a regular factory because I want to be able to order priority, etc
'''

from . import MonitoringList
from .ActivityFile import ActivityFileMonitor
from .RSSLimit import RSSLimitMonitor

from ..logging import logErr


ALL_MONITORING_CLASSES = [ActivityFileMonitor, RSSLimitMonitor]


class MonitoringFactory(object):

    @staticmethod
    def getAsyncMonitorsForProgram(programConfig):
        '''
            getMonitorsForProgram - Gets any async monitors (runs inline in the main monitoring thread) for a given program.

            @param programConfig <usrsvcmod.ProgramConfig obj> - The ProgramConfig for this program

            @return - list<MonitoringBase> - A list of monitors in the order they should be tried for the given program, or empty list if all monitors are disabled.
        '''
        ret = MonitoringList()

        # Right now we will just do a list, but we may need more complicated ordering in the future
        for monitoringClass in ALL_MONITORING_CLASSES:
            try:
                monitoringObj = monitoringClass.createFromConfig(programConfig)
                if monitoringObj is None:
                    continue
                ret.append(monitoringObj)
            except Exception as e:
                logErr('EXCEPTION while generating Monitoring classes! Class was %s and program was %s. Error: %s\n' %(monitoringClass.__name__, programConfig.name, str(e)))

        return ret




'''
    Copyright (c) 2016 Tim Savannah All Rights Reserved.
    This software is licensed under the terms of the GPLv3.
    This may change at my discretion, retroactively, and without notice.

    You should have received a copy of this with the source distribution as a file titled, LICENSE.

    The most current license can be found at:
    https://github.com/kata198/usrsvc/LICENSE

    This location may need to be changed at some point in the future, in which case
    you are may email Tim Savannah <kata198 at gmail dot com>, or find them on the
    current website intended for distribution of usrsvc.


    RSSLimitMonitor - Monitor maximum RSS (Resident Set Size)
'''

from func_timeout import FunctionTimedOut

from . import MonitoringBase
from ..logging import logMsg, logErr



try:
    import resource
    PAGE_SIZE = resource.getpagesize()
except:
    # Fallback if resource is not available, assume 4096.
    PAGE_SIZE = 4096

class RSSLimitMonitor(MonitoringBase):
    '''
       RSSLimitMonitor - Class for monitoring RSS
    '''

    def __init__(self, programName, rssLimit):
        '''
            @param programName <str> - Name of program
            @param rssLimit <int> - kB maximum RSS size
        '''
        self.programName = programName
        self.rssLimit = rssLimit

    @classmethod
    def createFromConfig(cls, programConfig):
        if not programConfig.Monitoring.rss_limit:
            return None

        return cls(programConfig.name, programConfig.Monitoring.rss_limit)

    def shouldRestart(self, program):
        rssLimit = self.rssLimit

        if rssLimit <= 0:
            # Yes this is checked twice if created through createFromConfig, but it may be called otherwise so better safe.
            return False

        try:
            with open('/proc/%d/statm' %(program.pid,), 'rt') as f:
                contents = f.read()
            fields = contents.split()
            rssPages = int(fields[1])
            rssKB = int((rssPages * PAGE_SIZE) / 1024)
            if rssKB > self.rssLimit:
                logMsg('Restarting %s because RSS size %dkB exceeds limit of %dkB' %(self.programName, rssKB, rssLimit))
                return True
        except FunctionTimedOut:
            logErr('MONITOR: RSSLimit timed out on %s\n' %(self.programName,))
            raise
        except Exception as e:
            # If we got an exception, just log and try again next round.
            logErr('Got an exception in RSS monitoring. Not restarting program. Program="%s"\n%s\nlocals: %s\n' %(self.programName, str(program), str(locals())))

        return False 





#!/usr/bin/env python
'''
    Copyright (c) 2016 Tim Savannah All Rights Reserved.
    This software is licensed under the terms of the GPLv2.

    You should have received a copy of this with the source distribution as LICENSE,
    otherwise the most up to date license can be found at
    https://github.com/kata198/usrsvc/LICENSE

    A user service manager for running/managing/monitoring daemons and services
'''

import os
import sys
from setuptools import setup




if __name__ == '__main__':

    dirName = os.path.dirname(__file__)
    if dirName and os.getcwd() != dirName:
        os.chdir(dirName)

    summary = 'A user service manager for running/managing/monitoring daemons and services'
    try:
        with open('README.rst', 'rt') as f:
            long_description = f.read()
    except Exception as e:
        sys.stderr.write('Error reading from README.rst: %s\n' %(str(e),))
        long_description = summary

    try:
        with open('usrsvcmod/client/_readme_contents.py', 'wt') as f:
            f.write('README_CONTENTS = """%s"""\n' %(long_description,)  )
            f.flush()
            try:
                os.fsync(f)
            except:
                pass
    except Exception as e:
        sys.stderr.write('Warning, failed to update readme.py - extended help will not be available.\n')
        sys.stderr.write(str(e)  +  '\n')

    setup(name='usrsvc',
            version='1.3.0',
            packages=['usrsvcmod', 'usrsvcmod.Monitoring', 'usrsvcmod.client'],
            scripts=['usrsvc', 'usrsvcd'],
            author='Tim Savannah',
            author_email='kata198@gmail.com',
            maintainer='Tim Savannah',
            url='https://github.com/kata198/usrsvc',
            maintainer_email='kata198@gmail.com',
            requires=['configobj', 'NamedAtomicLock', 'func_timeout'],
            install_requires=['configobj', 'NamedAtomicLock', 'func_timeout'],
            description=summary,
            long_description=long_description,
            license='GPLv2',
            keywords=['usrsvc', 'usrsvcd', 'daemon', 'user', 'services', 'service', 'init', 'script', 'start', 'stop', 'restart', 'manage', 'programs', 'applications', 'supervisor', 'supervisord', 'systemd', 'daemontools'],
            classifiers=['Development Status :: 4 - Beta',
                         'Programming Language :: Python',
                         'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
                         'Programming Language :: Python :: 2',
                          'Programming Language :: Python :: 2',
                          'Programming Language :: Python :: 2.7',
                          'Programming Language :: Python :: 3',
                          'Programming Language :: Python :: 3.3',
                          'Programming Language :: Python :: 3.4',
                          'Programming Language :: Python :: 3.5',
                          'Topic :: System :: Boot :: Init',
            ]
    )






'''
    Copyright (c) 2016 Tim Savannah All Rights Reserved.
    This software is licensed under the terms of the GPLv3.
    This may change at my discretion, retroactively, and without notice.

    You should have received a copy of this with the source distribution as a file titled, LICENSE.

    The most current license can be found at:
    https://github.com/kata198/usrsvc/LICENSE

    This location may need to be changed at some point in the future, in which case
    you are may email Tim Savannah <kata198 at gmail dot com>, or find them on the
    current website intended for distribution of usrsvc.


    constants for usrsvc
'''

try:
    from enum import IntEnum
except ImportError:
    # We only support 2.7+ anyway, but some older 2.7s don't have the enum type.
    IntEnum = object

class ReturnCodes(IntEnum):
    
    # A general success
    SUCCESS = 0

    # A general failure
    GENERAL_FAILURE = 1

    TRY_AGAIN = 11  # Same as EAGAIN on Linux. Should try operation again.

    #### Specific meanings ###

    # Config does not parse or is otherwise invalid
    INVALID_CONFIG  = 130

    # An action was requested that does not exist
    INVALID_ACTION  = 131

    # The program was disabled
    PROGRAM_DISABLED = 132

    # The program requested is not defined in configs
    PROGRAM_UNDEFINED = 133

    # Program was launched, but died before #success_seconds# passed.
    PROGRAM_EXITED_UNEXPECTEDLY = 134

    # Usrsvcd was started but another instance is already running as current user
    USRSVCD_ALREADY_RUNNING = 135

    # Insufficient permissions to perform the action (usually file permissions)
    INSUFFICIENT_PERMISSIONS = 136

    # Called when an invalid command or if cannot execute given command
    PROGRAM_FAILED_TO_LAUNCH = 137

    # Returned when a help option (like  --help or  --readme) is used
    HELP_MESSAGE = 138
    
    # Unknown exception occured, state of program is unknown.
    UNKNOWN_FAILURE = 254

    if IntEnum != object:
        # Preferred method using enum
        @classmethod
        def returnCodeToString(cls, returnCodeValue):
            try:
                returnCodeValue = int(returnCodeValue)
            except ValueError:
                # Should be an exception, but let's go for the more durable route.
                return 'INVALID_RETURN(%s)' %(str(returnCodeValue),)
                #raise ValueError('returnCodeToString called with non-integer value: %s' %(str(returnCodeValue),))

            for item in cls:
                if item.value == returnCodeValue:
                    return item.name

            return 'UNKNOWN(%d)' %(returnCodeValue,)
    else:
        # Backup method for older pythons without enum
        @classmethod
        def returnCodeToString(cls, returnCodeValue):
            try:
                returnCodeValue = int(returnCodeValue)
            except ValueError:
                return 'INVALID_RETURN(%s)' %(str(returnCodeValue),)

            for attrName in dir(cls):
                attrValue = getattr(cls, attrName)
                if not isinstance(attrValue, int):
                    continue
                if attrValue == returnCodeValue:
                    return attrName

            return 'UNKNOWN(%d)' %(returnCodeValue,)




'''
    Copyright (c) 2016 Tim Savannah All Rights Reserved.
    This software is licensed under the terms of the GPLv3.
    This may change at my discretion, retroactively, and without notice.

    You should have received a copy of this with the source distribution as a file titled, LICENSE.

    The most current license can be found at:
    https://github.com/kata198/usrsvc/LICENSE

    This location may need to be changed at some point in the future, in which case
    you are may email Tim Savannah <kata198 at gmail dot com>, or find them on the
    current website intended for distribution of usrsvc.


    'MonitoringConfig' represents the monitoring subsection of a ProgramConfig
'''

from .configcommon import getConfigValueBool, getConfigValueInt, getConfigValueFloat

__all__ = ('MonitoringConfig',)

class MonitoringConfig(object):
    '''
        MonitoringConfig - The [[Monitor]] subsection of a Program
    '''

    def __init__(self, monitor_after=30,
        activityfile='', activityfile_limit=120,
        rss_limit=0,
        **kwargs):
        '''
            Config values:

            monitor_after - Minimum number of seconds that program needs to be running before monitoring will begin. Default 30. 0 disables this feature.
            activityfile - File or Directory which must be modified every #activityfile_limit# seconds, or program will be restarted. Default undefined/empty string disables this.
            activityfile_limit - Default 120. If activityfile is defined, this is the number of seconds is the maximum that can go between modifications of the provided #activityfile# before triggering a restart.
            rss_limit - Default 0. If > 0, specifies the maximum RSS (Resident Set Size) in kilobytes (1024 bytes)
        '''

        self.monitor_after = getConfigValueInt(monitor_after, 'monitor_after')

        self.activityfile = activityfile
        if activityfile and activityfile[0] != '/':
            raise ValueError('activityfile must be an absolute path.\n')
        self.activityfile_limit = getConfigValueInt(activityfile_limit, 'activityfile_limit')

        self.rss_limit = getConfigValueInt(rss_limit, 'rss_limit')
        if self.rss_limit < 0:
            raise ValueError('rss_limit must be 0 to disable, or a positive integer for maximum kB of Resident Set Size')

        if kwargs:
            raise ValueError('Unknown configuration options in Monitoring section: %s' %(str(list(kwargs.keys())),))


    def isMonitoringActive(self):
        # "or" of all the various monitoring types.
        return bool(self.activityfile) or bool(self.rss_limit)

        
    def __str__(self):
        return str(self.__dict__)



'''
    Copyright (c) 2016 Tim Savannah All Rights Reserved.
    This software is licensed under the terms of the GPLv3.
    This may change at my discretion, retroactively, and without notice.

    You should have received a copy of this with the source distribution as a file titled, LICENSE.

    The most current license can be found at:
    https://github.com/kata198/usrsvc/LICENSE

    This location may need to be changed at some point in the future, in which case
    you are may email Tim Savannah <kata198 at gmail dot com>, or find them on the
    current website intended for distribution of usrsvc.


    Monitoring Stuff
'''

import time

from ..logging import logErr

__all__ = ('MonitoringBase', 'MonitoringList')

# TODO: Some monitoring will need to run as a separate thread and this base class should provide the means to launch and then later join and assert.
class MonitoringBase(object):
    '''
        MonitoringBase - A base class for monitoring. Right now mostly just a placeholder,
            but will be used when more advanced monitors are created.
    '''


    @classmethod
    def createFromConfig(cls, programConfig):
        '''
            createFromConfig - Create an instance of the monitoring class from the given config. Return None if this monitor is inactive for this program.

            @param programConfig <usrsvcmod.ProgramConfig>   - The ProgramConfig for a given program.

            @return <None/object> - None if the monitor is disabled, otherwise an instance of the monitoring class with all config values set.
        '''
        raise NotImplementedError('Monitor %s does not implement createFromConfig.' %(str(cls.__name__), ))

    def shouldRestart(self, program):
        '''
            shouldRestart - Run the test, and check if we should restart the app.

            @param program <usrsvcmod.Program> - The Program instance representing the running program being monitored. May not be required for all checks

            @return <bool> - True if we should trigger a restart.
        '''
        return False


class MonitoringList(list):
    '''
        MonitoringList - A list of monitors. Has functions to simplify common usage
    '''

    def executeList(self, program):
        '''
            executeList - Runs the monitors in order until all have been completed or one fails.

            @param program <usrsvcmod.Program> - The Program instance representing the Program to monitor

            @return <dict> - {
                'doRestart' : <bool> - True if we should trigger a restart, otherwise False.
                'triggeredAlert' : <None/object> - The object of the monitor that triggered
                'runtime' : <float> - The time it took to run the monitors
                'numRan'  : <int> - The number of monitors that ran
            }
        '''
        ret = {
            'doRestart' : False,
            'triggeredAlert' : None,
            'runtime' : time.time(),
            'numRan' : 0
        }

        shouldRestart = False
        for mon in self:
            ret['numRan'] += 1
            try:
                shouldRestart = mon.shouldRestart(program)
                if shouldRestart is True:
                    break
            except Exception as e: 
                logErr('Unexpected exception when running list of monitors: %s.\nLocals: %s\nMoving on..\n' %(str(e), str(locals())))
                continue

        if shouldRestart is True:
            ret['triggeredAlert'] = mon
            ret['doRestart'] = True

        ret['runtime'] = time.time() - ret['runtime']

        return ret
                



'''
    Copyright (c) 2016 Tim Savannah All Rights Reserved.
    This software is licensed under the terms of the GPLv3.
    This may change at my discretion, retroactively, and without notice.

    You should have received a copy of this with the source distribution as a file titled, LICENSE.

    The most current license can be found at:
    https://github.com/kata198/usrsvc/LICENSE

    This location may need to be changed at some point in the future, in which case
    you are may email Tim Savannah <kata198 at gmail dot com>, or find them on the
    current website intended for distribution of usrsvc.


    'ProgramConfig' represents a single program config
'''

import os
import re
import shlex

from .configcommon import getConfigValueBool, getConfigValueInt, getConfigValueFloat
from .MonitoringConfig import MonitoringConfig

__all__ = ('ProgramConfig', )

class ProgramConfig(object):
    '''
        ProgramConfig - The configuration for a Program. [Program:TheName] would be named "TheName"
    '''

    def __init__(self, name, 
            command=None, pidfile=None, autostart=True,
            autorestart=True, maxrestarts=0, restart_delay=0,
            autopid=True, useshell=True, proctitle_re=None, 
            success_seconds=2.0, term_to_kill_seconds=8.0, scan_for_process=True,
            stdout=None, stderr=None,
            enabled=True,
            inherit_env=True, Env=None, Monitoring=None,
            defaults=None,
            **kwargs):
        '''
            name - Determined from section name [Program:TheName] has "TheName"

            Params:

                * command - REQUIRED - Full command and arguments to execute. If #useshell# is True, this can contain shell-isms
                * useshell - If True, will invoke your application through a shell. You can use shell expressions in this mode. Use "False" if you don't need this.
                * pidfile - REQUIRED - Path to a pidfile. If #autopid# is False, your app must write its pid to this file. Otherwise, usrsvcd will mangage it, even with #scan_for_process# or other methods.
                * enabled - Boolean, default True. Set to "False" to disable the program from being managed by "usrsvcd"
                * autopid - Default True, boolean. If True, "usrsvc" and "usrsvcd" will write the pid of the launched program to the pidfile, i.e. managed. If your application forks-and-exits, you can set this to FAlse and write your own pid, or use #scan_for_process#
                * scan_for_process - Default True, boolean. If True, "usrsvc" and "usrsvcd" will, in the absense of a pidfile which matches with #proctitle_re#, use #proctitle_re# and scan running processes for the application. This can find applications even when the pidfile has gone missing.
                * proctitle_re - None or a regular expression which will match the proctitle (can be seen as last col in "ps auxww").  If none provided, a default wherein the command and arguments are used, will work in almost all instances. Some applications modify their proctitle, and you may need to use this to match them.

            
                * autostart - Default True, boolean value if program should be started if not already running when "usrsvcd" is invoked
                * autorestart - Default True, boolean value if program should be restarted if it stopped while "usrsvcd" is running
                * maxrestarts - Default 0, integer on the max number of times usrsvcd will try to automatically restart the application by "usrsvcd". If it is seen running again naturally, this counter will reset. 0 means unlimited restarts.
                * restart_delay - Default 0, integer on the miminum number of seconds between a failing "start" and the next "restart" attmept by "usrsvcd". 
                * success_seconds - Default 2, Float on the number of seconds the application must be running for "usrsvc" to consider it successfully started.
                * term_to_kill_seconds : Default 8, Float on the number of seconds the application is given between SIGTERM and SIGKILL.



                NOTE: The following stdout/stderr are opened in "append" mode always. 
                * stdout - REQUIRED - Absolute path to a file to be used for stdout
                * stderr - Absolute path to a file to be used for stderr, or "stdout" to redirect to stdout. Default is to redirect stderr to stdout. May be same filename as stdout.
                * defaults - This can reference a "DefaultSettings" section defined elsewhere, i.e. to reference [DefaultSettings:MySettings] use "defaults=MySettings". If provided, this Program will inherit the settings defined in the DefaultSettings as the defaults. Anything provided explicitly in this Program will override those found in the defaults.
                * inherit_env - Boolean, default True. If True, will inherit the env from "usrsvc" or "usrsvcd". Otherwise, will only use the Env as defined in the Env subsection.

                Supports the following subsections:

                [[Monitor]]
                  - See MonitoringConfig

                [[Env]]
                  key=value

                pairs for environment variables. Affected by #inherit_env above.
        '''


        self.command = command
        self.pidfile = pidfile
        if not pidfile or pidfile[0] != '/':
            raise ValueError('pidfile must be defined for a program and must be an absolute path.')
        self.enabled = getConfigValueBool(enabled, 'enabled')
        self.autostart = getConfigValueBool(autostart, 'autostart')
        self.autorestart = getConfigValueBool(autorestart, 'autorestart')
        self.maxrestarts = getConfigValueInt(maxrestarts, 'maxrestarts')
        self.restart_delay = getConfigValueInt(restart_delay, 'restart_delay')
        self.success_seconds = getConfigValueFloat(success_seconds, 'success_seconds')
        self.autopid = getConfigValueBool(autopid, 'autopid')
        self.useshell = getConfigValueBool(useshell, 'useshell')
        self.scan_for_process = getConfigValueBool(scan_for_process, 'scan_for_process')
        self.term_to_kill_seconds = getConfigValueFloat(term_to_kill_seconds, 'term_to_kill_seconds')

        if self.term_to_kill_seconds < 0:
            raise ValueError('term_to_kill_seconds is required and must be a positive number.')

        self.inherit_env = getConfigValueBool(inherit_env, 'inherit_env')
        self.defaults = defaults
        
        if not issubclass(Env.__class__, (type(None), dict)):
            raise ValueError('Env must be a subsection, like [[Env]]')

        if Env is None:
            Env = {}
        self.Env = Env

        if not issubclass(Monitoring.__class__, (type(None), dict)):
            raise ValueError('Monitoring must be a subsection, like [[Monitoring]]')
        if Monitoring is None:
            Monitoring = {}
        self.Monitoring = MonitoringConfig(**Monitoring)


        if not name:
            raise ValueError('Program Config defined without a name.')
        self.name = name

        if not command:
            raise ValueError('Missing command for program: "%s"' %(name,))

        try:
            commandSplit = shlex.split(command)
        except ValueError as e:
            raise ValueError('Cannot parse command,\n%s\nError: %s' %(command, str(e)))

        # If proctitle_re is not defined, default to the provided command.
        #   Do not use start because given a shebang line, the executable may change.
        if not proctitle_re:
            proctitle_re = re.compile('(%s)$' %(re.escape(' '.join(commandSplit)), ))

        self.proctitle_re = proctitle_re
            
        if not stdout:
            raise ValueError('Program %s does not define a value for stdout.' %(name,))
        elif stdout[0] != '/':
            raise ValueError('Program %s has invalid stdout path "%s", must be absolute.' %(name, stdout))

        if not os.path.isdir(os.path.dirname(stdout)):
            raise ValueError('Program %s defines a stdout path as "%s" but the folder %s does not exist.' %(name, stdout, os.path.dirname(stdout)))

        self.stdout = stdout

        if not stderr or stderr == 'stdout':
            self.stderr = stdout
        else:
            if stderr[0] != '/':
                raise ValueError('Program %s has invalid stderr path "%s", must be absolute.' %(name, stderr))
            if not os.path.isdir(os.path.dirname(stderr)):
                raise ValueError('Program %s defines a stderr path as "%s" but the folder %s does not exist.' %(name, stderr, os.path.dirname(stderr)))

            self.stderr = stderr

        if kwargs:
            raise ValueError('Unknown config options: %s' %(str(list(kwargs.keys())), ))

    def __str__(self):
        return str(self.__dict__)
